<?php

class RCCart {

	public function __construct () {
		
		$this->getID();
		
		$this->loadReferrer();
		$this->loadCouponCode();
		$this->loadItems();
		$this->loadCustomer();
		$this->loadOptions();
		$this->loadShipping();
		$this->loadPayment();
		
		$this->setTaxes();
		$this->setTotal();
		
	}
	
	private function getID ($reset = FALSE) {
		
		$ID = strtoupper(uniqid());
		
		$this->ID = $reset ? RCUtilities::saveSessionData('ID', $ID) : RCUtilities::loadSessionData('ID', $ID);
		
	}
	
	private function loadReferrer () {
		
		$this->referrer = RCUtilities::loadSessionData('referrer', '');
		
	}
	
	public function setReferrer ($referrer) {
		
		$this->referrer = RCUtilities::saveSessionData('referrer', $referrer);
		
	}
	
	public function addItem ($product) {
		
		$quantity = $product->selectedQuantity;
		unset($product->selectedQuantity);
	
		$ID = md5(serialize($product));
		
		$found = FALSE;
	
		foreach ($this->items as $item) {
		
			if ($item->ID == $ID) {
				
				if ($item->product->enableStockManagement) {
					
					$item->quantity = min($item->quantity + $quantity, $item->product->stockQuantity, $item->product->quantity);
					
				}
				
				else {
					
					$item->quantity = ($item->product->quantity > 0) ? min($item->quantity + $quantity, $item->product->quantity) : $item->quantity + $quantity;
					
				}
				
				$found = TRUE;
				break;
				
			}
			
		}
		
		if (!$found) {
				
			if ($product->enableStockManagement) {
				
				$quantity = min($quantity, $product->stockQuantity, $product->quantity);
				
			}
			
			else {
				
				$quantity = ($product->quantity > 0) ? min($quantity, $product->quantity) : $quantity;
				
			}
	
			array_push($this->items, (object) array(
			
				'ID'       => $ID,
				'quantity' => $quantity,
				'product'  => $product
				
			));
		
		}
		
		$this->saveCart();
		$this->setTotal();
		
	}
	
	public function updateQuantity ($ID, $quantity) {
		
		if (!is_numeric($quantity)) {
			
			return;
			
		}
		
		if ($quantity < 1) {
			
			$this->removeItem($ID);
			return;
			
		}
		
		foreach ($this->items as $item) {
		
			if ($item->ID == $ID) {
				
				if ($item->product->enableStockManagement) {
					
					$item->quantity = ($item->product->quantity == 0) ? min($quantity, $item->product->stockQuantity) : min($quantity, $item->product->stockQuantity, $item->product->quantity);
					
				}
				
				else {
					
					$item->quantity = ($item->product->quantity > 0) ? min($quantity, $item->product->quantity) : $quantity;
					
				}
				
				return;
				
			}
		
		}
		
	}
	
	public function removeItem ($ID) {
		
		foreach ($this->items as $i => $item) {
			
			if ($item->ID == $ID) {
				
				unset($this->items[$i]);
				
			}
			
		}
		
		$this->items = array_values($this->items);
		
		$this->saveCart();
		
	}
	
	public function removeItems ($items) {
		
		foreach ($items as $ID) {
			
			$this->removeItem($ID);
			
		}
		
	}
	
	private function loadCouponCode () {
		
		$this->coupon = FALSE;
	
		if (RCUtilities::$settings->cart->acceptCouponCodes) {
			
			$this->applyCouponCode(RCUtilities::loadSessionData('coupon', FALSE));
			
		}
		
	}
	
	public function applyCouponCode ($coupon) {
		
		$this->coupon = FALSE;
		
		$now = strtotime('now');
		
		if (RCUtilities::$settings->cart->acceptCouponCodes) {
			
			foreach (RCUtilities::$settings->cart->costs as $cost) {
				
				if (
					isset($cost->couponRequired) &&
					$cost->couponRequired &&
					trim(strtoupper($coupon)) == trim($cost->couponCode) &&
					$cost->couponLimit > 0 &&
					(!$cost->validity->startEnabled || $now > strtotime($cost->validity->start)) &&
					(!$cost->validity->endEnabled || $now < strtotime($cost->validity->end))
				) {
					
					$this->coupon = trim(strtoupper($coupon));
					
				}
				
			}
			
		}
		
		RCUtilities::saveSessionData('coupon', $this->coupon);
		
	}
	
	public function removeCouponCode () {
		
		$this->applyCouponCode(FALSE);
		
	}
	
	private function loadItems () {
	
		$this->items = RCUtilities::loadSessionData('items', array());
		
		$this->shippable = FALSE;
		
		foreach ($this->items as $item) {
			
			if ($item->product->shippable) {
				
				$this->shippable = TRUE;
				
			}
			
		}
		
	}
	
	private function saveCart () {
		
		RCUtilities::saveSessionData('items', $this->items);
		
	}
	
	private function setTotal () {
		
		$this->count = 0;
		$this->weight = 0;
		$this->subtotal = 0;
		$this->total = 0;
		$this->taxes = 0;
		$this->defaultTaxes = 0;
		
		if (empty($this->items)) {
			
			return;
			
		}
		
		foreach ($this->items as $item) {
			
			$this->count += $item->quantity;
			$this->weight += $item->product->weight * $item->quantity;
			$this->subtotal += $item->product->price * $item->quantity;
			$this->taxes += $item->product->taxable ? ($item->product->price * $item->quantity * $this->taxesRates[$item->product->taxesTier] / 100) : 0;
			
			$this->defaultTaxes += $item->product->taxable ? ((isset($item->product->discountedPrice) ? $item->product->discountedPrice : $item->product->price) * $item->quantity * RCUtilities::$settings->product->defaultTaxes[$item->product->taxesTier] / 100) : 0;
			
			if ($item->product->shippable) {
				
				$this->shippable = TRUE;
				
			}
			
		}
		
		$this->addCosts();
		$this->loadDealer();
				
		if (RCUtilities::$settings->cart->applyTaxesToCosts) {
			
			$this->taxes += $this->costs->total * $this->taxesRates[0] / 100;
			
		}
				
		if (RCUtilities::$settings->cart->applyTaxesToOptions) {
			
			$this->taxes += $this->options->price * $this->taxesRates[0] / 100;
			
		}
				
		if (RCUtilities::$settings->cart->applyTaxesToShipping) {
			
			$this->taxes += $this->shipping->price * $this->taxesRates[0] / 100;
			
		}
				
		if (RCUtilities::$settings->cart->applyTaxesToPayment) {
			
			$this->taxes += $this->payment->price * $this->taxesRates[0] / 100;
			
		}
		
		$this->subtotal += $this->costs->total;
		$this->total = $this->subtotal + $this->shipping->price + $this->options->price + $this->payment->price + $this->taxes;
		
	}
	
	private function addCosts () {
		
		$this->costs = (object) array(
			
			'total' => 0,
			'list'  => array()
			
		);
			
		$now = strtotime('now');
		
		foreach (RCUtilities::$settings->cart->costs as $cost) {
			
			if (
			
				($cost->validity->startEnabled && $now < strtotime($cost->validity->start)) ||
				($cost->validity->endEnabled && $now > strtotime($cost->validity->end)) ||
				(isset($cost->couponRequired) && $cost->couponRequired && ($cost->couponLimit == 0 || $cost->couponCode != $this->coupon))
				
			) {
				
				continue;
				
			}
			
			$match = FALSE;
			$price = 0;
				
			switch ($cost->type) {
				
				case 'cart':
					
					$price = $this->subtotal;
					$match = TRUE;
					
				break;
				
				case 'product':
			
					foreach ($this->items as $item) {
				
						if (in_array($item->product->SKU, $cost->values)) {
					
							$price += $item->product->price * $item->quantity;
							$match = TRUE;
							
						}
						
					}
				
				break;
				
				case 'total':
				
					foreach ($cost->steps as $step) {
						
						if ($this->subtotal <= $step->value) {
							
							$price = $this->subtotal;
							$cost->price = $step->price;
							$match = TRUE;
							break;
							
						}
						
					}
				
				break;
				
				case 'items':
				
					foreach ($cost->steps as $step) {
						
						if ($this->count <= $step->value) {
							
							$price = $this->subtotal;
							$cost->price = $step->price;
							$match = TRUE;
							break;
							
						}
						
					}
				
				break;
				
				case 'weight':
				
					foreach ($cost->steps as $step) {
						
						if ($this->weight <= $step->value) {
							
							$price = $this->subtotal;
							$cost->price = $step->price;
							$match = TRUE;
							break;
							
						}
						
					}
					
				break;
				
				case 'tag':
			
					foreach ($this->items as $item) {
						
						$array = array_intersect($item->product->tags, $cost->values);
				
						if (!empty($array)) {
							
							$price += $item->product->price * $item->quantity;
							$match = TRUE;
							
						}
						
					}
					
				break;
				
				case 'category':
			
					foreach ($this->items as $item) {
						
						$array = array_intersect($item->product->categories, $cost->values);
						
						if (!empty($array)) {
							
							$price += $item->product->price * $item->quantity;
							$match = TRUE;
							
						}
						
					}
					
				break;
				
			}
			
			if ($match && $cost->price != 0) {
				
				$price = ($cost->mode == 'fixed') ? $cost->price : ($price * $cost->price / 100);
				
				$this->costs->total += $price;
				
				$this->costs->list[] = (object) array(
					
					'description' => $cost->name,
					'price'       => $price
					
				);
				
			}
			
		}
		
	}

	private function loadDealer () {

		if (RCUtilities::$settings->dealers->loggedIn) {

			if (RCUtilities::$settings->dealers->applyTo == 'all' || RCUtilities::$settings->dealers->applyTo == 'products') {

				$price = 0 - $this->subtotal / 100 * RCUtilities::$settings->dealers->discount;
				
				$this->costs->total += $price;
				
				$this->costs->list[] = (object) array(
					
					'description' => RCUtilities::$localization->dealers->discountMessage,
					'price'       => $price
					
				);

			}

			if (RCUtilities::$settings->dealers->applyTo == 'all' || RCUtilities::$settings->dealers->applyTo == 'shipping') {

				$this->shipping->price = 0;

			}

			if (RCUtilities::$settings->dealers->applyTo == 'all' || RCUtilities::$settings->dealers->applyTo == 'payment') {

				$this->payment->price = 0;

			}

		}

	}
	
	private function loadCustomer () {
		
		$customer = (object) array(
			
			'name'          => '',
			'emailAddress'  => '',
			'country'       => '',
			'countryCode'   => '',
			'company'       => '',
			'VATID'         => '',
			'VATIDVerified' => FALSE,
			'street'        => '',
			'ZIPCode'       => '',
			'city'          => '',
			'state'         => '',
			'stateCode'     => '',
			'phone'         => '',
			'customField1'  => '',
			'customField2'  => '',
			'customField3'  => '',
			'customField4'  => '',
			'customField5'  => '',
			'customField6'  => '',
			'customField7'  => '',
			'customField8'  => '',
			'customField9'  => '',
			'customField10' => '',
			'valid'         => FALSE
			
		);
	
		$this->customer = RCUtilities::loadSessionData('customer', $customer);
		
	}
	
	private function saveCustomer ($data) {
		
		RCUtilities::saveSessionData('customer', $data);
		
	}
	
	public function validateCustomerDetails ($data) {
		
		$required = array();
		
		if (empty($data['rc-cart-customer-name'])) {
			
			$required[] = 'rc-cart-customer-name';
			
		}
		
		if (!filter_var($data['rc-cart-customer-email-address'], FILTER_VALIDATE_EMAIL) || RCUtilities::isBlacklisted($_POST['rc-cart-customer-email-address']) !== TRUE) {
			
			$required[] = 'rc-cart-customer-email-address';
			
		}
		
		if (RCUtilities::$settings->cart->customerRequireRepeatEmail && ($data['rc-cart-customer-email-address'] != $data['rc-cart-customer-email-address-2'] || empty($data['rc-cart-customer-email-address-2']))) {
			
			$required[] = 'rc-cart-customer-email-address';
			$required[] = 'rc-cart-customer-email-address-2';
			
		}
		
		if (RCUtilities::$settings->cart->customerCompany == 'require' && empty($data['rc-cart-customer-company'])) {
			
			$required[] = 'rc-cart-customer-company';
			
		}
		
		if (RCUtilities::$settings->cart->customerVATID == 'require' && empty($data['rc-cart-customer-vat-id'])) {
			
			$required[] = 'rc-cart-customer-vat-id';
			
		}
		
		$VATIDVerified = FALSE;
		
		if (!empty($data['rc-cart-customer-vat-id']) && RCUtilities::$settings->cart->validateVATID) {
			
			try {
				
				@ini_set('default_socket_timeout', 3);
			
				$client = new SoapClient('http://ec.europa.eu/taxation_customs/vies/checkVatService.wsdl');
				
				$result = $client->checkVat(array(
				
					'countryCode' => $data['rc-cart-customer-country'],
					'vatNumber'   => $data['rc-cart-customer-vat-id']
					
				));
				
			}
			
			catch (Exception $client) {
				
				$result = (object) array(
					
					'valid' => TRUE
					
				);
				
			}
			
			$VATIDVerified = $result->valid;
			
			if (!$result->valid) {
			
				$required[] = 'rc-cart-customer-vat-id';
				
			}
			
		}
		
		if (RCUtilities::$settings->cart->customerAddress == 'require') {
		
			if (empty($data['rc-cart-customer-street'])) {
				
				$required[] = 'rc-cart-customer-street';
				
			}
		
			if (empty($data['rc-cart-customer-zip-code'])) {
				
				$required[] = 'rc-cart-customer-zip-code';
				
			}
		
			if (empty($data['rc-cart-customer-city'])) {
				
				$required[] = 'rc-cart-customer-city';
				
			}
			
		}
		
		if (RCUtilities::$settings->cart->customerPhone == 'require' && empty($data['rc-cart-customer-phone'])) {
			
			$required[] = 'rc-cart-customer-phone';
			
		}
		
		if (RCUtilities::$settings->cart->customerCustomField1 == 'require' && empty($data['rc-cart-customer-custom-field-1'])) {
			
			$required[] = 'rc-cart-customer-custom-field-1';
			
		}
		
		if (RCUtilities::$settings->cart->customerCustomField2 == 'require' && empty($data['rc-cart-customer-custom-field-2'])) {
			
			$required[] = 'rc-cart-customer-custom-field-2';
			
		}
		
		if (RCUtilities::$settings->cart->customerCustomField3 == 'require' && empty($data['rc-cart-customer-custom-field-3'])) {
			
			$required[] = 'rc-cart-customer-custom-field-3';
			
		}
		
		if (RCUtilities::$settings->cart->customerCustomField4 == 'require' && empty($data['rc-cart-customer-custom-field-4'])) {
			
			$required[] = 'rc-cart-customer-custom-field-4';
			
		}
		
		if (RCUtilities::$settings->cart->customerCustomField5 == 'require' && empty($data['rc-cart-customer-custom-field-5'])) {
			
			$required[] = 'rc-cart-customer-custom-field-5';
			
		}
		
		if (RCUtilities::$settings->cart->customerCustomField6 == 'require' && empty($data['rc-cart-customer-custom-field-6'])) {
			
			$required[] = 'rc-cart-customer-custom-field-6';
			
		}
		
		if (RCUtilities::$settings->cart->customerCustomField7 == 'require' && empty($data['rc-cart-customer-custom-field-7'])) {
			
			$required[] = 'rc-cart-customer-custom-field-7';
			
		}
		
		if (RCUtilities::$settings->cart->customerCustomField8 == 'require' && empty($data['rc-cart-customer-custom-field-8'])) {
			
			$required[] = 'rc-cart-customer-custom-field-8';
			
		}
		
		if (RCUtilities::$settings->cart->customerCustomField9 == 'require' && empty($data['rc-cart-customer-custom-field-9'])) {
			
			$required[] = 'rc-cart-customer-custom-field-9';
			
		}
		
		if (RCUtilities::$settings->cart->customerCustomField10 == 'require' && empty($data['rc-cart-customer-custom-field-10'])) {
			
			$required[] = 'rc-cart-customer-custom-field-10';
			
		}
		
		if (empty($required)) {
			
			$data = (object) array(
				
				'name'          => $data['rc-cart-customer-name'],
				'emailAddress'  => $data['rc-cart-customer-email-address'],
				'country'       => $this->getCountry($data['rc-cart-customer-country']),
				'countryCode'   => $data['rc-cart-customer-country'],
				'company'       => isset($data['rc-cart-customer-company']) ? $data['rc-cart-customer-company'] : '',
				'VATID'         => isset($data['rc-cart-customer-vat-id']) ? $data['rc-cart-customer-vat-id'] : '',
				'VATIDVerified' => $VATIDVerified,
				'street'        => isset($data['rc-cart-customer-street']) ? $data['rc-cart-customer-street'] : '',
				'ZIPCode'       => isset($data['rc-cart-customer-zip-code']) ? $data['rc-cart-customer-zip-code'] : '',
				'city'          => isset($data['rc-cart-customer-city']) ? $data['rc-cart-customer-city'] : '',
				'state'         => isset($data['rc-cart-customer-state']) ? $this->getState($data['rc-cart-customer-state']) : '',
				'stateCode'     => isset($data['rc-cart-customer-state']) ? $data['rc-cart-customer-state'] : '',
				'phone'         => isset($data['rc-cart-customer-phone']) ? $data['rc-cart-customer-phone'] : '',
				'customField1'  => isset($data['rc-cart-customer-custom-field-1']) ? $data['rc-cart-customer-custom-field-1'] : '',
				'customField2'  => isset($data['rc-cart-customer-custom-field-2']) ? $data['rc-cart-customer-custom-field-2'] : '',
				'customField3'  => isset($data['rc-cart-customer-custom-field-3']) ? $data['rc-cart-customer-custom-field-3'] : '',
				'customField4'  => isset($data['rc-cart-customer-custom-field-4']) ? $data['rc-cart-customer-custom-field-4'] : '',
				'customField5'  => isset($data['rc-cart-customer-custom-field-5']) ? $data['rc-cart-customer-custom-field-5'] : '',
				'customField6'  => isset($data['rc-cart-customer-custom-field-6']) ? $data['rc-cart-customer-custom-field-6'] : '',
				'customField7'  => isset($data['rc-cart-customer-custom-field-7']) ? $data['rc-cart-customer-custom-field-7'] : '',
				'customField8'  => isset($data['rc-cart-customer-custom-field-8']) ? $data['rc-cart-customer-custom-field-8'] : '',
				'customField9'  => isset($data['rc-cart-customer-custom-field-9']) ? $data['rc-cart-customer-custom-field-9'] : '',
				'customField10' => isset($data['rc-cart-customer-custom-field-10']) ? $data['rc-cart-customer-custom-field-10'] : '',
				'valid'         => TRUE
			
			);
			
			$this->saveCustomer($data);
			
		}
		
		return $required;
		
	}
	
	private function loadOptions () {
		
		$this->options = (object) array(
			
			'price'  => 0,
			'weight' => 0,
			'set'    => array(),
			'valid'  => FALSE
			
		);
		
		foreach (RCUtilities::$settings->cart->optionSets as $set) {
				
			if ($set->type == 'cart') {
				
				$this->options->set = array_merge($this->options->set, $set->options);
				
			}
			
		}
	
		$this->validateOptions(RCUtilities::loadSessionData('options', array()));
		
	}
	
	public function validateOptions ($options) {
			
		$this->saveOptions($options);
		
		$required = array();
		
		foreach ($this->options->set as $option) {
		
			// date, time, textarea, text, number
			if (in_array($option->type, array(
			
				'text',
				'textarea',
				'date',
				'time',
				'number'
			
			))) {
				
				$option->values = isset($options[$option->ID]) ? $options[$option->ID] : array('');
					
				if (isset($option->multiplier) && isset($options[$option->ID])) {
					
					$this->options->price += $options[$option->ID][0] * $option->multiplier;
					
				}
				
				if (
				
					($option->required && empty($options[$option->ID][0])) ||
					($option->type == 'number' && (
						
						(isset($option->min) && (empty($options[$option->ID][0]) || $options[$option->ID][0] < $option->min)) ||
						(isset($option->max) && (empty($options[$option->ID][0]) || $options[$option->ID][0] > $option->max))
						
					))
					
				) {
					
					$required[] = $option->ID;
					
				}
				
			}
			
			// select, checkbox, radio
			else {
				
				$tempRequired = array();
				
				$j = 0;
				
				foreach ($option->values as $i => $value) {
					
					$tempRequired[] = $value->ID;
					
					$value->selected = FALSE;
					
					if (isset($options[$option->ID]) && in_array($value->ID, $options[$option->ID])) {
						
						$value->selected = TRUE;
						
						$this->options->weight += $value->weight;
						$this->options->price += $value->price;
						
						$j = $i;
						
					}
					
				}
				
				if ($option->required) {
					
					if ($option->type == 'select' && $j == 0) {
						
						$required[] = $option->ID;
						
					}
					
					else if (empty($options[$option->ID])) {
					
						$required = array_merge($required, $tempRequired);
						
					}
					
				}
				
			}
			
		}
		
		if (empty($required)) {
			
			$this->options->valid = TRUE;
			
		}
		
		return $required;
		
	}
	
	private function saveOptions ($data) {
		
		RCUtilities::saveSessionData('options', $data);
		
	}
	
	private function getCountry ($code) {
		
		foreach (RCUtilities::$settings->cart->allowedCountries as $country) {
			
			if ($country->code == $code) {
				
				return $country->name;
				
			}
			
		}
		
		return code;
		
	}
	
	private function getState ($code) {
		
		foreach (RCUtilities::$settings->cart->allowedCountries as $country) {
 			
 			if (isset($country->states)) {
	 			
	 			foreach ($country->states as $state) {
		 			
		 			if ($state->code == $code) {
			 			
			 			return $state->name;
			 			
		 			}
		 			
	 			}
	 			
	 		}
	 		
	 	}
	 			
	 	return $code;
		
	}
	
	private function setTaxes () {
		
		$this->taxesRates = array(0, 0, 0);
		
		if (!empty($this->customer->VATID) && (RCUtilities::$settings->cart->EUVATCountry != $this->customer->countryCode)) {
			
			return;
			
		}
		
		if (isset(RCUtilities::$settings->cart->allowedCountries)) {
			
			foreach (RCUtilities::$settings->cart->allowedCountries as $country) {
				
				if ($country->code == $this->customer->countryCode) {
					
					$this->taxesRates = array(isset($country->taxes) ? $country->taxes : 0, isset($country->taxes2) ? $country->taxes2 : 0, isset($country->taxes3) ? $country->taxes3 : 0);
					
				}
				
				if (isset($country->states)) {
					
					foreach ($country->states as $state) {
					
						if ($state->code == $this->customer->stateCode) {
							
							$this->taxesRates = array(isset($state->taxes) ? $state->taxes : 0, isset($state->taxes2) ? $state->taxes2 : 0, isset($state->taxes3) ? $state->taxes3 : 0);
							
						}
						
					}
					
				}
				
			}
			
		}
		
	}
	
	public function getShippingMethods () {
		
		$methods = array();
			
		foreach (RCUtilities::$settings->cart->shipping as $i => $method) {
			
			if (empty($method->countries) || in_array($this->customer->countryCode, $method->countries) || in_array($this->customer->stateCode, $method->countries)) {
				
				if ((isset($method->freeOverAmountEnabled) && $method->freeOverAmountEnabled) && (($this->subtotal + $this->options->price) >= $method->freeOverAmount)) {
					
					$price = 0;
					
				}
				
				else {
					
					switch ($method->type) {
						
						case 'total':
						
							$value = $this->subtotal + $this->options->price;
							
						break;
						
						case 'weight':
						
							$value = $this->weight + $this->options->weight;
							
						break;
						
						case 'items':
						
							$value = $this->count;
							
						break;
						
					}
					
					$price = 0;
					
					foreach ($method->steps as $step) {
						
						if ($value <= $step->value) {
							
							$price = $step->price;
							break;
							
						}
						
					}
				
				}
				
				$methods[] = (object) array(
					
					'name'     => $method->name,
					'price'    => ($method->mode == 'fixed') ? $price : (($this->subtotal + $this->options->price) * $price / 100),
					'selected' => ($i == $this->shipping->ID) || (empty($this->shipping->ID) && $i == 0)
					
				);
			
			}
			
		}
		
		return $methods;
		
	}
	
	public function validateShippingMethod ($data) {
		
		if (isset($data['rc-cart-shipping'])) {
			
			$ID = $data['rc-cart-shipping'];
			
		}
		
		else {
			
			return FALSE;
			
		}
		
		$methods = $this->getShippingMethods();
		
		if (isset($methods[$ID])) {
			
			$shipping = (object) array(
				
				'ID'     => $ID,
				'name'   => $methods[$ID]->name,
				'price'  => $methods[$ID]->price,
				'valid'  => TRUE
				
			);
			
			$this->saveShipping($shipping);
			
			return TRUE;
			
		}
		
		return FALSE;
		
	}
	
	private function saveShipping ($data) {
		
		RCUtilities::saveSessionData('shipping', $data);
		
	}
	
	private function loadShipping () {
		
		$shipping = (object) array(
			
			'ID'     => '',
			'name'   => '',
			'price'  => 0,
			'valid'  => FALSE
			
		);
	
		$this->shipping = $this->shippable ? RCUtilities::loadSessionData('shipping', $shipping) : $shipping;
		
	}
	
	public function getPaymentMethods () {
		
		$payments = array();
		
		foreach (RCUtilities::$settings->payments as $payment) {
			
			if ($payment->enable && (empty($payment->countries) || in_array($this->customer->countryCode, $payment->countries))) {
				
				switch ($payment->type) {
					
					case 'paypal':
					
						if ($this->total == 0) {
							
							continue 2;
							
						}
					
						$payment->testMode = $payment->sandbox;
						
					break;
					
					case 'stripe':
					
						if ($this->total == 0) {
							
							continue 2;
							
						}
					
						$payment->testMode = strpos(RCUtilities::$config->stripe->APIKey, 'sk_test_') === 0;
						
					break;
					
					case 'mollie':
					
						if ($this->total == 0) {
							
							continue 2;
							
						}
					
						$payment->testMode = strpos(RCUtilities::$config->mollie->APIKey, 'test_') === 0;
						
					break;
					
					default:
					
						if (isset($payment->freeOrdersOnly) && $payment->freeOrdersOnly && $this->total != 0) {
							
							continue 2;
							
						}
					
						$payment->testMode = FALSE;
						
					break;
					
				}
				
				$payments[] = $payment;
				
			}
			
		}
		
		usort($payments, function ($a, $b) {
				
			return $a->position - $b->position;
			
		});
		
		foreach ($payments as $i => $payment) {
				
			$payment->selected = (!isset($this->payment->method) && $i == 0) || (isset($this->payment->method) && $payment->type == $this->payment->method->type);
			$payment->price = ($payment->commissionMode == 'fixed') ? $payment->commission : (($this->subtotal + $this->options->price + $this->shipping->price) * $payment->commission / 100);
			
		}
		
		return $payments;
		
	}
	
	public function setPaymentMethod ($ID) {
		
		foreach ($this->getPaymentMethods() as $payment) {
			
			if ($ID == $payment->type) {
				
				$this->payment = (object) array(
					
					'method' => $payment,
					'price'  => $payment->price,
					'valid'  => TRUE
					
				);
				
				$this->savePayment();
				
			}
			
		}
		
	}
	
	private function savePayment () {
		
		RCUtilities::saveSessionData('payment', $this->payment);
		
	}
	
	public function setCreditCardNumber ($number) {
		
		$this->payment->method->creditCardNumber = empty($number) ? '' : ('•••• •••• •••• ' . substr($number, -4));
		
		$this->savePayment();
		
	}
	
	private function loadPayment () {
		
		$payment = (object) array(
			
			'price' => 0,
			'valid' => FALSE
			
		);
	
		$this->payment = RCUtilities::loadSessionData('payment', $payment);
		
	}
	
	public function saveTransactionID ($ID) {
		
		$this->payment->transactionID = $ID;
		
		$this->savePayment();
		
	}
	
	public function reset () {
		
		$this->getID(TRUE);
		
		$this->removeCouponCode();
		
		$this->items = array();
		$this->saveCart();
		
		$this->customer->valid = FALSE;
		$this->saveCustomer($this->customer);
		
		$this->saveOptions(array());
		
		$this->shipping->valid = FALSE;
		$this->saveShipping($this->shipping);
		
		$this->payment->valid = FALSE;
		$this->payment->method->creditCardNumber = '';
		$this->savePayment();
		
		$this->saveTransactionID('');
		
	}
	
	public function getSections ($active) {
		
		$sections = array();
		
		$sections[] = (object) array(
			
			'name' => 'overview',
			'icon' => 'shopping-basket'
			
		);
		
		$sections[] = (object) array(
			
			'name' => 'address',
			'icon' => 'user'
			
		);
		
		if (!empty($this->options->set)) {
			
			$sections[] = (object) array(
				
				'name' => 'options',
				'icon' => 'paperclip'
				
			);
			
		}
		
		if ($this->shippable) {
			
			$sections[] = (object) array(
				
				'name' => 'shipping',
				'icon' => 'truck'
				
			);
			
		}
		
		$sections[] = (object) array(
			
			'name' => 'payment',
			'icon' => 'credit-card'
			
		);
		
		$sections[] = (object) array(
			
			'name' => 'review',
			'icon' => 'list-ul'
			
		);
		
		$sections[] = (object) array(
			
			'name' => 'success',
			'icon' => 'check'
			
		);
		
		$history = array();
		
		foreach ($sections as $i => $section) {
			
			$section->active = $section->name == $active;
			
			if (!$section->active && !in_array($active, $history)) {
				
				$section->link = RCUtilities::getURL(array(
				
					0 => 'cart',
					1 => $section->name
				
				), TRUE);
				
			}
			
			$history[] = $section->name;
			
		}
		
		return $sections;
		
	}
	
	public function generatesDownloadLinks () {
		
		$time = (string) time();
		$download = FALSE;
		
		foreach ($this->items as $item) {
			
			if ($item->product->downloadable) {
				
				$download = TRUE;
				
				$item->downloadLink = RCUtilities::getURL(array(
				
					0 => 'download',
					1 => urlencode(base64_encode(RCUtilities::dataEncrypt($time))),
					2 => urlencode(base64_encode(RCUtilities::dataEncrypt($item->product->SKU))),
					3 => urlencode(base64_encode(RCUtilities::dataEncrypt($this->customer->emailAddress)))
				
				), TRUE);
				
			}
			
		}
		
		return $download;
		
	}

}

?>