<?php

class RCDatabase {

	public function __construct () {
		
		$this->error = '';
		
		try {

			$this->pdo = new PDO(
				sprintf('mysql:dbname=%s;host=%s;port=%u',
					RCUtilities::$config->database->name,
					RCUtilities::$config->database->host,
					RCUtilities::$config->database->port
				),
				RCUtilities::$config->database->username,
				RCUtilities::$config->database->password,
				array(
					PDO::ATTR_TIMEOUT => 5,
					PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION
				)
			);
			
			$this->version = $this->pdo->getAttribute(constant("PDO::ATTR_SERVER_VERSION"));
			
			$this->checkTables();

		}
		
		catch (PDOException $e) {
			
			$this->error = 'Connection failed: ' . $e->getMessage();
		
		}
		
	}
	
	private function checkTables () {
		
		try {
		
			$this->pdo->exec('CREATE TABLE IF NOT EXISTS `Products` (
				`SKU` varchar(32) NOT NULL,
				`Quantity` int(11) NOT NULL,
				`LastUpdate` datetime NOT NULL,
				`Synced` tinyint(1) NOT NULL DEFAULT 1,
				PRIMARY KEY (`SKU`)
			)');
		
			$this->pdo->exec('CREATE TABLE IF NOT EXISTS `Coupons` (
				`Code` varchar(32) NOT NULL,
				`Quantity` int(11) NOT NULL,
				`LastUpdate` datetime NOT NULL,
				`Synced` tinyint(1) NOT NULL DEFAULT 1,
				PRIMARY KEY (`Code`)
			)');
			
			$this->pdo->exec('CREATE TABLE IF NOT EXISTS `Downloads` (
				`Token` varchar(32) NOT NULL,
				`SKU` varchar(32) NOT NULL,
				`Email` varchar(64) NOT NULL,
				`Date` datetime NOT NULL,
				`Downloads` int(11) NOT NULL,
				PRIMARY KEY (`Token`, `SKU`)
			)');
			
			$this->pdo->exec('CREATE TABLE IF NOT EXISTS `Orders` (
				`ID` varchar(16) NOT NULL,
				`Date` datetime NOT NULL,
				`CustomerName` varchar(64) NOT NULL,
				`CustomerEmail` varchar(64) NOT NULL,
				`CustomerCountry` varchar(64) NOT NULL,
				`CustomerCountryCode` varchar(2) NOT NULL,
				`CustomerCompany` varchar(64) NOT NULL,
				`CustomerVATID` varchar(16) NOT NULL,
				`CustomerVATIDVerified` tinyint(1) NOT NULL,
				`CustomerStreet` varchar(64) NOT NULL,
				`CustomerZIPCode` varchar(16) NOT NULL,
				`CustomerCity` varchar(64) NOT NULL,
				`CustomerState` varchar(64) NOT NULL,
				`CustomerPhone` varchar(32) NOT NULL,
				`CustomerCustomField1` varchar(64) NOT NULL,
				`CustomerCustomField2` varchar(64) NOT NULL,
				`CustomerCustomField3` varchar(64) NOT NULL,
				`CustomerCustomField4` varchar(64) NOT NULL,
				`CustomerCustomField5` varchar(64) NOT NULL,
				`CustomerCustomField6` varchar(64) NOT NULL,
				`CustomerCustomField7` varchar(64) NOT NULL,
				`CustomerCustomField8` varchar(64) NOT NULL,
				`CustomerCustomField9` text NOT NULL,
				`CustomerCustomField10` text NOT NULL,
				`CustomerIP` varchar(64) NOT NULL,
				`Items` text NOT NULL,
				`Options` text NOT NULL,
				`ShippingName` varchar(64) NOT NULL,
				`ShippingPrice` float NOT NULL,
				`PaymentType` varchar(32) NOT NULL,
				`PaymentName` varchar(64) NOT NULL,
				`PaymentPrice` float NOT NULL,
				`TransactionID` varchar(64) NOT NULL,
				`Costs` text NOT NULL,
				`Coupon` varchar(32) NOT NULL,
				`Weight` float NOT NULL,
				`Shippable` tinyint(1) NOT NULL,
				`Subtotal` float NOT NULL,
				`Taxes` float NOT NULL,
				`Total` float NOT NULL,
				`SubtotalRefunded` float NOT NULL,
				`TaxesRefunded` float NOT NULL,
				`TotalRefunded` float NOT NULL,
				`Status` varchar(16) NOT NULL,
				`Referrer` varchar(16) NOT NULL,
				`Notes` text NOT NULL,
				PRIMARY KEY (`ID`)
			)');
			
			$this->pdo->exec('CREATE TABLE IF NOT EXISTS `Ratings` (
				`Order` varchar(16) NOT NULL,
				`Product` int(11) NOT NULL,
				`Rate` int(1) NOT NULL,
				PRIMARY KEY (`Order`,`Product`)
			)');
			
			// fix per supporto IPv6
			$select = $this->pdo->query('SHOW COLUMNS FROM `Orders` WHERE `Field`= "CustomerIP"', PDO::FETCH_COLUMN, 1);
			
			if ($select->fetch() == 'varchar(32)') {
				
				$this->pdo->exec('ALTER TABLE `Orders` MODIFY `CustomerIP` varchar(64)');
				
			}
			
			// fix per tipo colonna Orders.ShippingPrice int(11) invece di float
			$select = $this->pdo->query('SHOW COLUMNS FROM `Orders` WHERE `Field`= "ShippingPrice"', PDO::FETCH_COLUMN, 1);
				
			if ($select->fetch() == 'int(11)') {
				
				$this->pdo->exec('ALTER TABLE `Orders` MODIFY `ShippingPrice` float, MODIFY `PaymentPrice` float, MODIFY `Weight` float');
				
			}
			
			// fix per tipo colonna Orders.Status int(16) invece di varchar(16)
			$select = $this->pdo->query('SHOW COLUMNS FROM `Orders` WHERE `Field`= "Status"', PDO::FETCH_COLUMN, 1);
				
			if ($select->fetch() == 'int(16)') {
				
				$this->pdo->exec('ALTER TABLE `Orders` MODIFY `Status` varchar(16)');
				$this->pdo->exec('UPDATE `Orders` SET `Status` = "completed"');
				
			}
			
			// fix per estrazione JSON cliente
			$select = $this->pdo->query('SHOW COLUMNS FROM `Orders` WHERE `Field`= "CustomerName"', PDO::FETCH_COLUMN, 0);
			
			if ($select->fetch() == '') {
				
				$this->pdo->exec('ALTER TABLE `Orders`
					ADD COLUMN `CustomerName` varchar(64) NOT NULL AFTER `Date`,
					ADD COLUMN `CustomerEmail` varchar(64) NOT NULL AFTER `CustomerName`,
					ADD COLUMN `CustomerCountry` varchar(64) NOT NULL AFTER `CustomerEmail`,
					ADD COLUMN `CustomerCountryCode` varchar(2) NOT NULL AFTER `CustomerCountry`,
					ADD COLUMN `CustomerCompany` varchar(64) NOT NULL AFTER `CustomerCountryCode`,
					ADD COLUMN `CustomerVATID` varchar(16) NOT NULL AFTER `CustomerCompany`,
					ADD COLUMN `CustomerVATIDVerified` tinyint(1) NOT NULL AFTER `CustomerVATID`,
					ADD COLUMN `CustomerStreet` varchar(64) NOT NULL AFTER `CustomerVATIDVerified`,
					ADD COLUMN `CustomerZIPCode` varchar(16) NOT NULL AFTER `CustomerStreet`,
					ADD COLUMN `CustomerCity` varchar(64) NOT NULL AFTER `CustomerZIPCode`,
					ADD COLUMN `CustomerState` varchar(64) NOT NULL AFTER `CustomerCity`,
					ADD COLUMN `CustomerIP` varchar(32) NOT NULL AFTER `CustomerState`
				');
					
				$update = $this->pdo->prepare('UPDATE `Orders` SET
					`CustomerName` = :CustomerName,
					`CustomerEmail` = :CustomerEmail,
					`CustomerCountry` = :CustomerCountry,
					`CustomerCountryCode` = :CustomerCountryCode,
					`CustomerCompany` = :CustomerCompany,
					`CustomerVATID` = :CustomerVATID,
					`CustomerVATIDVerified` = :CustomerVATIDVerified,
					`CustomerStreet` = :CustomerStreet,
					`CustomerZIPCode` = :CustomerZIPCode,
					`CustomerCity` = :CustomerCity,
					`CustomerState` = :CustomerState,
					`CustomerIP` = ""
					WHERE `ID` = :ID
				');
			
				$select = $this->pdo->query('SELECT `ID`, `Customer` FROM `Orders`');
				
				foreach ($select->fetchAll(PDO::FETCH_OBJ) as $row) {
					
					$customer = json_decode($row->Customer);
					
					$update->execute(array(
						
						':CustomerName'          => $customer->name,
						':CustomerEmail'         => $customer->emailAddress,
						':CustomerCountry'       => $customer->country,
						':CustomerCountryCode'   => $customer->countryCode,
						':CustomerCompany'       => $customer->company,
						':CustomerVATID'         => $customer->VATID,
						':CustomerVATIDVerified' => $customer->VATIDVerified ? 1 : 0,
						':CustomerStreet'        => $customer->street,
						':CustomerZIPCode'       => $customer->ZIPCode,
						':CustomerCity'          => $customer->city,
						':CustomerState'         => $customer->state,
						':ID'                    => $row->ID
						
					));
					
				}
				
				$this->pdo->exec('ALTER TABLE `Orders` DROP COLUMN `Customer`');
				$this->pdo->exec('UPDATE `Orders` SET `Status` = "to-ship" WHERE `Status` = "Completed" AND `Shippable` = 1');
				
			}
			
			// fix per nuovi campi custom
			$select = $this->pdo->query('SHOW COLUMNS FROM `Orders` WHERE `Field`= "CustomerCustomField1"', PDO::FETCH_COLUMN, 0);
			
			if ($select->fetch() == '') {
				
				$this->pdo->exec('ALTER TABLE `Orders`
					ADD COLUMN `CustomerPhone` varchar(32) NOT NULL AFTER `CustomerState`,
					ADD COLUMN `CustomerCustomField1` varchar(64) NOT NULL AFTER `CustomerPhone`,
					ADD COLUMN `CustomerCustomField2` varchar(64) NOT NULL AFTER `CustomerCustomField1`,
					ADD COLUMN `CustomerCustomField3` varchar(64) NOT NULL AFTER `CustomerCustomField2`,
					ADD COLUMN `CustomerCustomField4` varchar(64) NOT NULL AFTER `CustomerCustomField3`,
					ADD COLUMN `CustomerCustomField5` varchar(64) NOT NULL AFTER `CustomerCustomField4`,
					ADD COLUMN `CustomerCustomField6` varchar(64) NOT NULL AFTER `CustomerCustomField5`,
					ADD COLUMN `CustomerCustomField7` varchar(64) NOT NULL AFTER `CustomerCustomField6`,
					ADD COLUMN `CustomerCustomField8` varchar(64) NOT NULL AFTER `CustomerCustomField7`,
					ADD COLUMN `CustomerCustomField9` text NOT NULL AFTER `CustomerCustomField8`,
					ADD COLUMN `CustomerCustomField10` text NOT NULL AFTER `CustomerCustomField9`
				');
				
			}
			
			// fix per estrazione JSON spedizione
			$select = $this->pdo->query('SHOW COLUMNS FROM `Orders` WHERE `Field`= "ShippingName"', PDO::FETCH_COLUMN, 0);
			
			if ($select->fetch() == '') {
				
				$this->pdo->exec('ALTER TABLE `Orders`
					ADD COLUMN `ShippingName` varchar(64) NOT NULL AFTER `Shipping`,
					ADD COLUMN `ShippingPrice` int(11) NOT NULL AFTER `ShippingName`
				');
					
				$update = $this->pdo->prepare('UPDATE `Orders` SET
					`ShippingName` = :ShippingName,
					`ShippingPrice` = :ShippingPrice
					WHERE `ID` = :ID
				');
			
				$select = $this->pdo->query('SELECT `ID`, `Shipping` FROM `Orders`');
				
				foreach ($select->fetchAll(PDO::FETCH_OBJ) as $row) {
					
					$shipping = json_decode($row->Shipping);
					
					$update->execute(array(
						
						':ShippingName'  => $shipping->name,
						':ShippingPrice' => $shipping->price,
						':ID'            => $row->ID
						
					));
					
				}
				
				$this->pdo->exec('ALTER TABLE `Orders` DROP COLUMN `Shipping`');
				
			}
			
			// fix per estrazione JSON pagamento
			$select = $this->pdo->query('SHOW COLUMNS FROM `Orders` WHERE `Field`= "PaymentType"', PDO::FETCH_COLUMN, 0);
			
			if ($select->fetch() == '') {
				
				$this->pdo->exec('ALTER TABLE `Orders`
					ADD COLUMN `PaymentType` varchar(32) NOT NULL AFTER `Payment`,
					ADD COLUMN `PaymentName` varchar(64) NOT NULL AFTER `PaymentType`,
					ADD COLUMN `PaymentPrice` int(11) NOT NULL AFTER `PaymentName`,
					ADD COLUMN `TransactionID` varchar(64) NOT NULL AFTER `PaymentPrice`
				');
					
				$update = $this->pdo->prepare('UPDATE `Orders` SET
					`PaymentType` = :PaymentType,
					`PaymentName` = :PaymentName,
					`PaymentPrice` = :PaymentPrice,
					`TransactionID` = :TransactionID
					WHERE `ID` = :ID
				');
			
				$select = $this->pdo->query('SELECT `ID`, `Payment` FROM `Orders`');
				
				foreach ($select->fetchAll(PDO::FETCH_OBJ) as $row) {
					
					$payment = json_decode($row->Payment);
					
					$update->execute(array(
						
						':PaymentType'   => $payment->method->type,
						':PaymentName'   => $payment->method->name,
						':PaymentPrice'  => $payment->price,
						':TransactionID' => $payment->transactionID,
						':ID'            => $row->ID
						
					));
					
				}
				
				$this->pdo->exec('ALTER TABLE `Orders` DROP COLUMN `Payment`');
				
			}
			
			// fix per inserimento referrer
			$select = $this->pdo->query('SHOW COLUMNS FROM `Orders` WHERE `Field`= "Referrer"', PDO::FETCH_COLUMN, 0);
			
			if ($select->fetch() == '') {
				
				$this->pdo->exec('ALTER TABLE `Orders`
					ADD COLUMN `Referrer` varchar(16) NOT NULL AFTER `Status`
				');
			
			}
			
		}
		
		catch (PDOException $e) {
			
			$this->error = 'Table error: ' . $e->getMessage();
		
		}
		
	}
	
	public function syncProducts (&$catalog, &$cart) {
		
		$select = $this->pdo->prepare('SELECT * FROM `Products` WHERE `SKU` = :SKU');
		
		$insert = $this->pdo->prepare('INSERT INTO `Products` (
			`SKU`,
			`Quantity`,
			`LastUpdate`,
			`Synced`
		) VALUES (
			:SKU,
			:Quantity,
			:LastUpdate,
			1
		)');
		
		$update = $this->pdo->prepare('UPDATE `Products` SET `Quantity` = :Quantity, `LastUpdate` = :LastUpdate, `Synced` = 1 WHERE `SKU` = :SKU');
		
		$this->pdo->exec('UPDATE `Products` SET `Synced` = 0');
		
		foreach ($catalog->products as $product) {
			
			foreach ($product->versions as $version) {
				
				if (!isset($version->enableStockManagement) || !$version->enableStockManagement) {
					
					continue;
					
				}
				
				$select->execute(array(
				
					':SKU' => $version->SKU
					
				));
				
				if (!$select->rowCount()) {
					
					$insert->execute(array(
						
						':SKU'        => $version->SKU,
						':Quantity'   => $version->stockQuantity,
						':LastUpdate' => $version->lastQuantityUpdate
						
					));
					
				}
				
				else {
					
					$row = $select->fetchObject();
					
					if ($row->LastUpdate == $version->lastQuantityUpdate) {
					
						$version->stockQuantity = $row->Quantity;
						
					}
					
					if ($row->Quantity == 0) {
						
						$version->available = FALSE;
						
					}
					
					foreach ($cart->items as $i => $item) {
						
						if ($item->product->SKU == $version->SKU) {
							
							if ($version->stockQuantity == 0) {
								
								unset($cart->items[$i]);
								
							}
							
							else {
							
								$item->product->stockQuantity = $version->stockQuantity;
								$item->quantity = min($item->quantity, $version->stockQuantity);
								
							}
							
							break;
							
						}
						
					} 
				
					$update->execute(array(
						
						':Quantity'   => $version->stockQuantity,
						':LastUpdate' => $version->lastQuantityUpdate,
						':SKU'        => $version->SKU
						
					));
					
				}
				
			}
			
		}
		
		$this->pdo->exec('DELETE FROM `Products` WHERE `Synced` = 0');
		
	}
	
	public function updateProducts (&$catalog, $cart) {
		
		$update = $this->pdo->prepare('UPDATE `Products` SET `Quantity` = GREATEST (0, `Quantity` - :Quantity) WHERE `SKU` = :SKU');
		
		foreach ($cart->items as $item) {
		
			$update->execute(array(
				
				':Quantity' => $item->quantity,
				':SKU'      => $item->product->SKU
				
			));
			
			foreach ($catalog->products as $product) {
				
				foreach ($product->versions as $version) {
					
					if ($item->product->SKU == $version->SKU && isset($version->stockQuantity)) {
						
						$version->stockQuantity = max(0, $version->stockQuantity - $item->quantity);
						
						break;
						
					}
					
				}
				
			}
			
		}
		
	}
	
	public function getRatings (&$catalog) {
		
		//$select = $this->pdo->query('SELECT `Product`, ROUND(ROUND(AVG(`Rate`) * 2) / 2, 1) AS `Rate`, COUNT(`Rate`) AS `Count` FROM `Ratings` GROUP BY `Product`');
		$select = $this->pdo->query('SELECT `Product`, ROUND(AVG(`Rate`)) AS `Rate`, COUNT(`Rate`) AS `Count` FROM `Ratings` GROUP BY `Product`');
		
		foreach ($select->fetchAll(PDO::FETCH_OBJ) as $ratings) {
			
			foreach ($catalog->products as $product) {
				
				if ($product->ID == $ratings->Product) {
					
					$product->ratingAverage = $ratings->Rate;
					$product->ratingCount = $ratings->Count;
					
				}
				
			}
			
		}
		
	}
	
	public function setRating ($order, $product, $rate) {
		
		$insert = $this->pdo->prepare('INSERT INTO `Ratings` (
			`Order`,
			`Product`,
			`Rate`
		) VALUES (
			:Order,
			:Product,
			:Rate
		) ON DUPLICATE KEY UPDATE `Rate` = :Rate');
		
		$insert->execute(array(
			
			':Order'   => $order,
			':Product' => $product,
			':Rate'    => $rate
			
		));
		
	}
	
	public function syncCoupons () {
		
		$select = $this->pdo->prepare('SELECT * FROM `Coupons` WHERE `Code` = :Code');
		
		$insert = $this->pdo->prepare('INSERT INTO `Coupons` (
			`Code`,
			`Quantity`,
			`LastUpdate`,
			`Synced`
		) VALUES (
			:Code,
			:Quantity,
			:LastUpdate,
			1
		)');
		
		$update = $this->pdo->prepare('UPDATE `Coupons` SET `Quantity` = :Quantity, `LastUpdate` = :LastUpdate, `Synced` = 1 WHERE `Code` = :Code');
		
		$this->pdo->exec('UPDATE `Coupons` SET `Synced` = 0');
		
		foreach (RCUtilities::$settings->cart->costs as $cost) {
			
			if (!isset($cost->couponRequired) || !$cost->couponRequired) {
				
				continue;
				
			}
			
			$select->execute(array(
			
				':Code' => $cost->couponCode
				
			));
			
			if (!$select->rowCount()) {
				
				if ($cost->couponLimit == 0) {
					
					$cost->couponLimit = 1;
					
				}
				
				else {
					
					$insert->execute(array(
						
						':Code'       => $cost->couponCode,
						':Quantity'   => $cost->couponLimit,
						':LastUpdate' => $cost->couponUpdate
						
					));
					
				}
				
			}
			
			else {
				
				$row = $select->fetchObject();
				
				$sync = TRUE;
				
				if ($row->LastUpdate == $cost->couponUpdate) {
				
					$cost->couponLimit = $row->Quantity;
					
				}
				
				else if ($cost->couponLimit == 0) {
					
					$cost->couponLimit = 1;
					
					$sync = FALSE;
					
				}
				
				if ($sync) {
				
					$update->execute(array(
						
						':Quantity'   => $cost->couponLimit,
						':LastUpdate' => $cost->couponUpdate,
						':Code'       => $cost->couponCode
						
					));
				
				}
				
			}
			
		}
		
		$this->pdo->exec('DELETE FROM `Coupons` WHERE `Synced` = 0');
		
	}
	
	public function updateCoupons ($code) {
		
		$update = $this->pdo->prepare('UPDATE `Coupons` SET `Quantity` = GREATEST (0, `Quantity` - 1) WHERE `Code` = :Code');
		
		$update->execute(array(
			
			':Code' => $code
			
		));
		
	}
	
	public function addDownloads (&$cart) {
		
		$token = md5(microtime());
		
		$insert = $this->pdo->prepare('INSERT INTO `Downloads` (
			`Token`,
			`SKU`,
			`Email`,
			`Date`,
			`Downloads`
		) VALUES (
			:Token,
			:SKU,
			:Email,
			UTC_TIMESTAMP(),
			0
		)');
		
		$download = FALSE;
		
		foreach ($cart->items as $item) {
			
			if ($item->product->downloadable) {
				
				$insert->execute(array(
					
					':Token' => $token,
					':SKU'   => $item->product->SKU,
					':Email' => $cart->customer->emailAddress
					
				));
				
				$item->downloadLink = RCUtilities::getURL(array(
				
					0 => 'download',
					1 => $token,
					2 => urlencode($item->product->SKU)
				
				), TRUE);
				
				$download = TRUE;
				
			}
			
		}
		
		return $download;
		
	}
	
	public function getDownload ($token, $SKU) {
		
		$select = $this->pdo->prepare('SELECT * FROM `Downloads` WHERE `Token` = :Token AND `SKU` = :SKU');
		
		$select->execute(array(
		
			':Token' => $token,
			':SKU'   => $SKU
			
		));
		
		if (!$select->rowCount()) {
			
			return FALSE;
			
		}
		
		$data = $select->fetchObject();
		
		return (object) array(
			
			'time'      => strtotime($data->Date),
			'email'     => $data->Email,
			'SKU'       => $data->SKU,
			'downloads' => $data->Downloads
			
		);
		
	}
	
	public function updateDownload ($token, $SKU) {
		
		$update = $this->pdo->prepare('UPDATE `Downloads` SET `Downloads` = `Downloads` + 1 WHERE `Token` = :Token AND `SKU` = :SKU');
		
		$update->execute(array(
			
			':Token' => $token,
			':SKU'   => $SKU
			
		));
		
	}
	
	public function addOrder ($cart) {
		
		$insert = $this->pdo->prepare('INSERT INTO `Orders` (
			`ID`,
			`Date`,
			`CustomerName`,
			`CustomerEmail`,
			`CustomerCountry`,
			`CustomerCountryCode`,
			`CustomerCompany`,
			`CustomerVATID`,
			`CustomerVATIDVerified`,
			`CustomerStreet`,
			`CustomerZIPCode`,
			`CustomerCity`,
			`CustomerState`,
			`CustomerPhone`,
			`CustomerCustomField1`,
			`CustomerCustomField2`,
			`CustomerCustomField3`,
			`CustomerCustomField4`,
			`CustomerCustomField5`,
			`CustomerCustomField6`,
			`CustomerCustomField7`,
			`CustomerCustomField8`,
			`CustomerCustomField9`,
			`CustomerCustomField10`,
			`CustomerIP`,
			`Items`,
			`Options`,
			`ShippingName`,
			`ShippingPrice`,
			`PaymentType`,
			`PaymentName`,
			`PaymentPrice`,
			`TransactionID`,
			`Costs`,
			`Coupon`,
			`Weight`,
			`Shippable`,
			`Subtotal`,
			`Taxes`,
			`Total`,
			`SubtotalRefunded`,
			`TaxesRefunded`,
			`TotalRefunded`,
			`Status`,
			`Referrer`,
			`Notes`
		) VALUES (
			:ID,
			UTC_TIMESTAMP(),
			:CustomerName,
			:CustomerEmail,
			:CustomerCountry,
			:CustomerCountryCode,
			:CustomerCompany,
			:CustomerVATID,
			:CustomerVATIDVerified,
			:CustomerStreet,
			:CustomerZIPCode,
			:CustomerCity,
			:CustomerState,
			:CustomerPhone,
			:CustomerCustomField1,
			:CustomerCustomField2,
			:CustomerCustomField3,
			:CustomerCustomField4,
			:CustomerCustomField5,
			:CustomerCustomField6,
			:CustomerCustomField7,
			:CustomerCustomField8,
			:CustomerCustomField9,
			:CustomerCustomField10,
			:CustomerIP,
			:Items,
			:Options,
			:ShippingName,
			:ShippingPrice,
			:PaymentType,
			:PaymentName,
			:PaymentPrice,
			:TransactionID,
			:Costs,
			:Coupon,
			:Weight,
			:Shippable,
			:Subtotal,
			:Taxes,
			:Total,
			0,
			0,
			0,
			:Status,
			:Referrer,
			""
		)');
		
		$insert->execute(array(
			
			':ID'                    => $cart->ID,
			':CustomerName'          => $cart->customer->name,
			':CustomerEmail'         => $cart->customer->emailAddress,
			':CustomerCountry'       => $cart->customer->country,
			':CustomerCountryCode'   => $cart->customer->countryCode,
			':CustomerCompany'       => $cart->customer->company,
			':CustomerVATID'         => $cart->customer->VATID,
			':CustomerVATIDVerified' => $cart->customer->VATIDVerified ? 1 : 0,
			':CustomerStreet'        => $cart->customer->street,
			':CustomerZIPCode'       => $cart->customer->ZIPCode,
			':CustomerCity'          => $cart->customer->city,
			':CustomerState'         => $cart->customer->state,
			':CustomerPhone'         => $cart->customer->phone,
			':CustomerCustomField1'  => $cart->customer->customField1,
			':CustomerCustomField2'  => $cart->customer->customField2,
			':CustomerCustomField3'  => $cart->customer->customField3,
			':CustomerCustomField4'  => $cart->customer->customField4,
			':CustomerCustomField5'  => $cart->customer->customField5,
			':CustomerCustomField6'  => $cart->customer->customField6,
			':CustomerCustomField7'  => $cart->customer->customField7,
			':CustomerCustomField8'  => $cart->customer->customField8,
			':CustomerCustomField9'  => $cart->customer->customField9,
			':CustomerCustomField10' => $cart->customer->customField10,
			':CustomerIP'            => $_SERVER['REMOTE_ADDR'],
			':Items'                 => json_encode($cart->items),
			':Options'               => json_encode($cart->options),
			':ShippingName'          => $cart->shipping->name,
			':ShippingPrice'         => $cart->shipping->price,
			':PaymentType'           => $cart->payment->method->type,
			':PaymentName'           => $cart->payment->method->name,
			':PaymentPrice'          => $cart->payment->price,
			':TransactionID'         => $cart->payment->transactionID,
			':Costs'                 => json_encode($cart->costs),
			':Coupon'                => $cart->coupon,
			':Weight'                => $cart->weight,
			':Shippable'             => $cart->shippable ? 1 : 0,
			':Subtotal'              => $cart->subtotal,
			':Taxes'                 => $cart->taxes,
			':Total'                 => $cart->total,
			':Status'                => $cart->shippable ? 'to-ship' : 'completed',
			':Referrer'              => $cart->referrer
			
		));
		
	}
	
	public function getOrders ($search, $status, $fromDate, $toDate, $start, $limit) {
		
		$query = 'SELECT * FROM `Orders` WHERE 1';
		
		if (!empty($search)) {
			
			$filter = explode(' ', $search);
			
			foreach ($filter as $token) {
				
				$token = explode(':', $token);
				
				if (count($token) > 1) {
					
					$value = trim(addslashes($token[1]));
					
					switch (strtolower($token[0])) {
						
						case 'id':
						
							$query .= ' AND `ID` = "' . $value . '"';
						
						break;
						case 'name':
						
							$query .= ' AND `CustomerName` LIKE "%' . $value . '%"';
							
						break;
						case 'email':
						
							$query .= ' AND `CustomerEmail` LIKE "%' . $value . '%"';
							
						break;
						case 'country':
						
							$query .= ' AND `CustomerCountry` LIKE "%' . $value . '%"';
							
						break;
						case 'company':
						
							$query .= ' AND `CustomerCompany` LIKE "%' . $value . '%"';
							
						break;
						case 'vat':
						
							$query .= ' AND `CustomerVATID` = "' . $value . '"';
							
						break;
						case 'sku':
						
							$query .= ' AND `Items` LIKE \'%"SKU":"' . $value . '"%\'';
						
						break;
						case 'payment':
						
							$query .= ' AND (`PaymentType` LIKE "%' . $value . '%" OR `PaymentName` LIKE "%' . $value . '%")';
						
						break;
						case 'coupon':
						
							$query .= ' AND `Coupon` = "' . $value . '"';
							
						break;
						case 'notes':
						
							$query .= ' AND `Notes` LIKE "%' . $value . '%"';
							
						break;
						case 'referrer':
						
							$query .= ' AND `Referrer` = "' . $value . '"';
							
						break;
						
					}
					
				}
				
				else {
					
					$value = trim(addslashes($token[0]));
					
					$query .= ' AND (
						`ID` = "' . $value . '" OR
						`CustomerName` LIKE "%' . $value . '%" OR
						`CustomerEmail` LIKE "%' . $value . '%" OR
						`CustomerCountry` LIKE "%' . $value . '%" OR
						`CustomerCompany` LIKE "%' . $value . '%" OR
						`CustomerVATID` = "' . $value . '" OR
						`Items` LIKE \'%"SKU":"' . $value . '"%\' OR
						`PaymentType` LIKE "%' . $value . '%" OR
						`PaymentName` LIKE "%' . $value . '%" OR
						`Coupon` = "' . $value . '" OR
						`Notes` LIKE "%' . $value . '%" OR
						`Referrer` = "' . $value . '"
						)';
					
				}
				
			}
			
		}
		
		switch ($status) {
			
			case 'completed':
			
				$query .= ' AND `Status` = "completed"';
				
			break;
			
			case 'to-ship':
			
				$query .= ' AND `Status` = "to-ship"';
				
			break;
			
			case 'refunded':
			
				$query .= ' AND `Status` = "refunded"';
				
			break;
			
			case 'cancelled':
			
				$query .= ' AND `Status` = "cancelled"';
				
			break;
			
		}
		
		if (!empty($fromDate)) {
			
			$fromDate = gmdate('Y-m-d H:i:s', strtotime($fromDate . ' 00:00:00'));
			$query .= ' AND `Date` >= :FromDate';
			
		}
		
		if (!empty($toDate)) {
			
			$toDate = gmdate('Y-m-d H:i:s', strtotime($toDate . ' 23:59:59'));
			$query .= ' AND `Date` <= :ToDate';
			
		}
		
		$selectCount = $this->pdo->prepare($query);
		
		$query .= ' ORDER BY `Date` DESC LIMIT :Limit OFFSET :Start';
		
		$select = $this->pdo->prepare($query);
		
		if (!empty($fromDate)) {
			
			$selectCount->bindParam(':FromDate', $fromDate);
			$select->bindParam(':FromDate', $fromDate);
			
		}
		
		if (!empty($toDate)) {
			
			$selectCount->bindParam(':ToDate', $toDate);
			$select->bindParam(':ToDate', $toDate);
			
		}
		
		$start = $start * $limit;
		
		$select->bindParam(':Start', $start, PDO::PARAM_INT);
		$select->bindParam(':Limit', $limit, PDO::PARAM_INT);
		
		$selectCount->execute();
		$select->execute();
		
		$total = 0;
		
		foreach ($selectCount->fetchAll(PDO::FETCH_OBJ) as $order) {
			
			$total += ($order->Status == 'cancelled') ? 0 : ($order->Total - $order->TotalRefunded);
			
		}
		
		$selectTotal = $this->pdo->prepare('SELECT COUNT(*) AS `Count`, COALESCE(SUM(`Total` - `TotalRefunded`), 0) AS `Total` FROM `Orders` WHERE
			`Date` >= :DateFrom AND
			`Date` <= :DateTo AND
			`Status` != "Cancelled"');
		
		$selectTotal->execute(array(
			
			':DateFrom' => gmdate('Y-m-d H:i:s', strtotime(date('Y-m-d 00:00:00'))),
			':DateTo'   => gmdate('Y-m-d H:i:s', strtotime(date('Y-m-d 23:59:59')))
			
		));
		
		$resultTotal = $selectTotal->fetchObject();
		
		$ordersToday = $resultTotal->Count;
		$totalToday = $resultTotal->Total;
		
		$selectTotal->execute(array(
			
			':DateFrom' => gmdate('Y-m-d H:i:s', strtotime(date('Y-m-d 00:00:00', strtotime('yesterday')))),
			':DateTo'   => gmdate('Y-m-d H:i:s', strtotime(date('Y-m-d 23:59:59', strtotime('yesterday'))))
			
		));
		
		$resultTotal = $selectTotal->fetchObject();
		
		$ordersYesterday = $resultTotal->Count;
		$totalYesterday = $resultTotal->Total;
		
		$selectTotal->execute(array(
			
			':DateFrom' => gmdate('Y-m-d H:i:s', strtotime(date('Y-m-01 00:00:00'))),
			':DateTo'   => gmdate('Y-m-d H:i:s', strtotime(date('Y-m-t 23:59:59')))
			
		));
		
		$resultTotal = $selectTotal->fetchObject();
		
		$ordersMonth = $resultTotal->Count;
		$totalMonth = $resultTotal->Total;
		
		$selectTotal->execute(array(
			
			':DateFrom' => gmdate('Y-m-d H:i:s', strtotime(date('Y-01-01 00:00:00'))),
			':DateTo'   => gmdate('Y-m-d H:i:s', strtotime(date('Y-12-31 23:59:59')))
			
		));
		
		$resultTotal = $selectTotal->fetchObject();
		
		$ordersYear = $resultTotal->Count;
		$totalYear = $resultTotal->Total;
		
		return (object) array(
		
			'count'           => $selectCount->rowCount(),
			'total'           => $total,
			'totalToday'      => $totalToday,
			'totalYesterday'  => $totalYesterday,
			'totalMonth'      => $totalMonth,
			'totalYear'       => $totalYear,
			'ordersToday'     => $ordersToday,
			'ordersYesterday' => $ordersYesterday,
			'ordersMonth'     => $ordersMonth,
			'ordersYear'      => $ordersYear,
			'orders'          => $select->fetchAll(PDO::FETCH_OBJ)
			
		);
		
	}
	
	public function getPendingOrdersCount () {
		
		$select = $this->pdo->prepare('SELECT * FROM `Orders` WHERE `Status` = "to-ship"');

		$select->execute();
		
		return $select->rowCount();
		
	}
	
	public function getOrder ($ID, $trueID = TRUE) {
		
		if ($trueID) {
			
			$select = $this->pdo->prepare('SELECT * FROM `Orders` WHERE `ID` = :ID');
			
		}
		
		else {
			
			$select = $this->pdo->prepare('SELECT * FROM `Orders` WHERE MD5(CONCAT(`ID`, `CustomerEmail`)) = :ID');
			
		}
			
		$select->execute(array(
			
			':ID' => $ID
			
		));
		
		$order = $select->fetchObject();
		
		if ($order !== FALSE) {
			
			$order->Items = json_decode($order->Items);
			$order->Costs = json_decode($order->Costs);
			
			$options = json_decode($order->Options);
			
			$order->Options = RCUtilities::optionsToList($options->set);
			$order->OptionsPrice = $options->price;
			
			$order->location = FALSE;
			
			if (!empty($order->CustomerIP)) {
				
				$ipinfo = json_decode(@file_get_contents('http://ipinfo.io/' . $order->CustomerIP . '/json'));
				
				if (isset($ipinfo->loc) && $ipinfo->hostname != 'localhost') {
					
					$order->location = $ipinfo->loc;
				}
				
			}
			
		}
		
		return $order;
		
	}
	
	public function updateOrder ($order) {
		
		if (!isset($order->subtotal)) {
			
			$order->subtotal = 0;
			
		}
		
		if (!isset($order->taxes)) {
			
			$order->taxes = 0;
			
		}
		
		$order->total = $order->subtotal + $order->taxes;
		
		$update = $this->pdo->prepare('UPDATE `Orders` SET
			`SubtotalRefunded` = :SubtotalRefunded,
			`TaxesRefunded` = :TaxesRefunded,
			`TotalRefunded` = :TotalRefunded,
			`Status` = :Status,
			`Notes` = :Notes
			WHERE `ID` = :ID
		');
		
		$update->execute(array(
			
			':SubtotalRefunded' => $order->subtotal,
			':TaxesRefunded'    => $order->taxes,
			':TotalRefunded'    => $order->total,
			':Status'           => $order->status,
			':Notes'            => $order->notes,
			'ID'                => $order->ID
			
		));
		
	}
	
	public function getStats () {
		
		$select = $this->pdo->prepare('SELECT COALESCE(SUM(`Total` - `TotalRefunded`), 0) AS `Total`, COUNT(*) AS `Orders` FROM `Orders` WHERE
			`Status` != "cancelled" AND
			`Date` >= :DateFrom AND
			`Date` <= :DateTo');
			
		$now = date('Y-m-d H:i:s');
		
		// trends today
		$trendsToday = array();
			
		for ($i = 0; $i < 5; $i++) {
			
			$select->execute(array(
				
				':DateFrom' => gmdate('Y-m-d H:i:s', strtotime(date('Y-m-d 00:00:00', strtotime($now . ' -' . $i . ' days')))),
				':DateTo'   => gmdate('Y-m-d H:i:s', strtotime(date('Y-m-d H:i:s', strtotime($now . ' -' . $i . ' days')))),
				
			));
			
			$trendsToday[] = $select->fetchObject();
			
		}
		
		// trends this month
		$trendsThisMonth = array();
			
		for ($i = 0; $i < 5; $i++) {
			
			$select->execute(array(
				
				':DateFrom' => gmdate('Y-m-d H:i:s', strtotime(date('Y-m-01 00:00:00', strtotime($now . ' -' . $i . ' months')))),
				':DateTo'   => gmdate('Y-m-d H:i:s', strtotime(date('Y-m-d H:i:s', strtotime($now . ' -' . $i . ' months')))),
				
			));
			
			$trendsThisMonth[] = $select->fetchObject();
			
		}
		
		// trends this year
		$trendsThisYear = array();
			
		for ($i = 0; $i < 5; $i++) {
			
			$select->execute(array(
				
				':DateFrom' => gmdate('Y-m-d H:i:s', strtotime(date('Y-01-01 00:00:00', strtotime($now . ' -' . $i . ' years')))),
				':DateTo'   => gmdate('Y-m-d H:i:s', strtotime(date('Y-m-d H:i:s', strtotime($now . ' -' . $i . ' years')))),
				
			));
			
			$trendsThisYear[] = $select->fetchObject();
			
		}
		
		// sales
		$sales = array();
			
		for ($i = 0; $i < 30; $i++) {
			
			$select->execute(array(
				
				':DateFrom' => gmdate('Y-m-d H:i:s', strtotime(date('Y-m-d 00:00:00', strtotime($now . ' -' . $i . ' days')))),
				':DateTo'   => gmdate('Y-m-d H:i:s', strtotime(date('Y-m-d 23:59:59', strtotime($now . ' -' . $i . ' days')))),
				
			));
			
			$sale = $select->fetchObject();
			
			$sale->Date = date('Y-m-d', strtotime($now . ' -' . $i . ' days'));
			$sale->TotalLocalized = RCUtilities::toCurrency($sale->Total, FALSE, FALSE);
			
			$sales[] = $sale;
			
		}
		
		//$select = $this->pdo->query('SELECT `CustomerCountry` AS `Name`, COALESCE(SUM(`Total`- `TotalRefunded`), 0) AS `Total`, COUNT(*) AS `Orders` FROM `Orders` WHERE `Status` != "cancelled" GROUP BY `CustomerCountry`');
		//$countries = $select->fetchAll(PDO::FETCH_OBJ);
		
		// best seller
		$select = $this->pdo->query('SELECT `Items` FROM `Orders` WHERE `Status` != "cancelled"');
		$bestSeller = array();
		
		foreach ($select->fetchAll(PDO::FETCH_OBJ) as $order) {
			
			$items = json_decode($order->Items);
			
			foreach ($items as $item) {
				
				$i = md5($item->product->name);
				
				if (isset($bestSeller[$i])) {
					
					$bestSeller[$i]->count += $item->quantity;
					$bestSeller[$i]->total += $item->product->price;
					
				}
				
				else {
					
					$bestSeller[$i] = (object) array(
						
						'name'  => $item->product->name,
						'count' => $item->quantity,
						'total' => $item->product->price
						
					);
					
				}
				
			}
			
		}
		
		usort($bestSeller, function ($a, $b) {
		
			return $b->count - $a->count;
			
		});
		
		$bestSeller = array_slice($bestSeller, 0, 3);
		
		foreach ($bestSeller as $product) {
			
			$product->totalLocalized = RCUtilities::toCurrency($product->total, FALSE, FALSE);
			
		}
		
		// best weekday
		$select = $this->pdo->query('SELECT DAYOFWEEK(`Date`) AS `Number`, COALESCE(SUM(`Total` - `TotalRefunded`), 0) AS `Total`, SUM(`Total` - `TotalRefunded`) / COUNT(DISTINCT(DATE(`Date`))) AS `Average` FROM `Orders` WHERE `Status` != "cancelled" GROUP BY DAYOFWEEK(`Date`)');
		$bestWeekdayRaw = $select->fetchAll(PDO::FETCH_OBJ);
		
		$bestWeekday = array();
		
		for ($i = 0; $i < 7; $i++) {
			
			$total = 0;
			$average = 0;
			
			foreach ($bestWeekdayRaw as $day) {
				
				if ($i == (($day->Number == 1) ? 6 : ($day->Number - 2))) {
					
					$total = $day->Total;
					$average = $day->Average;
					
				}
				
			}
			
			$bestWeekday[] = (object) array(
				
				'name'             => RCUtilities::$localization->dateTime->weekdays[$i],
				'total'            => $total,
				'totalLocalized'   => RCUtilities::toCurrency($total, FALSE, FALSE),
				'average'          => $average,
				'averageLocalized' => RCUtilities::toCurrency($average, FALSE, FALSE)
				
			);
			
		}
		
		return (object) array(
			
			'trendsToday'     => $trendsToday,
			'trendsThisMonth' => $trendsThisMonth,
			'trendsThisYear'  => $trendsThisYear,
			'sales'           => $sales,
			'bestSeller'      => $bestSeller,
			'bestWeekday'     => $bestWeekday
			
		);
		
	}
	
	public function updateAdminProducts ($products) {
		
		$update = $this->pdo->prepare('UPDATE `Products` SET `Quantity` = GREATEST (0, :Quantity) WHERE MD5(`SKU`) = :SKU');
		
		foreach ($products as $product) {
		
			$update->execute(array(
				
				':Quantity' => $product['quantity'],
				':SKU'      => $product['SKU']
				
			));
			
		}
		
	}
	
	public function updateAdminCoupons ($coupons) {
		
		$update = $this->pdo->prepare('UPDATE `Coupons` SET `Quantity` = GREATEST (0, :Quantity) WHERE MD5(`Code`) = :Code');
		
		foreach ($coupons as $coupon) {
		
			$update->execute(array(
				
				':Quantity' => $coupon['quantity'],
				':Code'     => $coupon['code']
				
			));
			
		}
		
	}
	
}