<?php

class RCProduct {

	public function __construct ($options) {
	
		foreach ($options as $key => $value) {
	
			$this->{$key} = $value;
		
		}
		
		$this->publishDate = strtotime($this->publishDate);
		
		$this->ratingAverage = 0;
		$this->ratingCount = 0;
		
		if (isset($this->pageURL) && !empty($this->pageURL)) {
			
			$this->link = $this->rel2abs($this->pageURL);
			$this->hasCustomPage = TRUE;
			
		}
		
		else {
			
			$this->link = RCUtilities::getURL(array(
			
				0 => 'product',
				1 => 'page',
				2 => $this->ID,
				3 => urlencode($this->name)
				
			), TRUE);
			$this->hasCustomPage = FALSE;
			
		}
		
		$this->buildResourcesSet();
		$this->addOptions();
		$this->setDiscounts();
		$this->setCategoriesAndTags();
		$this->setBasePriceAndTaxesTier();
		
		$this->validityID = md5(serialize($this->versions) . serialize($this->options));
		
		$this->requiredOptions = FALSE;
		
		foreach ($this->options as $option) {

			if ($option->required) {
				
				$this->requiredOptions = TRUE;
				break;
				
			}
			
		}
		
		$this->promoted = isset($this->promoted) && $this->promoted;
		
		$this->related = array();
		
	}
	
	private function buildResourcesSet () {
		
		foreach ($this->resources as $i => $resource) {
			
			$valid = FALSE;
			
			if (gettype($resource) == 'string') {
				
				$valid = TRUE;
				
				$thumbnailPath = $this->rel2abs($resource);
				$path          = $thumbnailPath;
				$type          = 'image';
				$caption       = '';
				
			}
			
			else if (!empty($resource->path)) {
				
				$valid = TRUE;
				
				$thumbnailPath = $this->rel2abs(!empty($resource->thumbnailPath) ? $resource->thumbnailPath : $resource->path);
				$path          = $this->rel2abs($resource->path);
				$type          = $resource->type;
				$caption       = isset($resource->caption) ? $resource->caption : '';
				
			}
			
			if ($valid) {
				
				$this->images[$i] = (object) array(
					
					'thumbnailPath' => $thumbnailPath,
					'path'          => $path,
					'type'          => $type,
					'caption'       => $caption
					
				);
			
			}
			
			else {
				
				echo $this->name . '<br>';
				
			}
			
		}
		
	}
	
	private function setCategoriesAndTags () {
		
		foreach ($this->categories as $i => $category) {
		
			$this->categories[$i] = (object) array(
			
				'ID'       => strtolower($category),
				'name'     => $category,
				'link'     => RCUtilities::getURL(array(
				
					0 => 'catalog',
					1 => 'category',
					2 => urlencode($category),
					3 => RCUtilities::$settings->catalog->defaultSort,
					4 => 1
					
				), TRUE)
				
			);
			
		}
		
		foreach ($this->tags as $i => $tag) {
		
			$this->tags[$i] = (object) array(
			
				'ID'       => strtolower($tag),
				'name'     => $tag,
				'link'     => RCUtilities::getURL(array(
				
					0 => 'catalog',
					1 => 'tag',
					2 => urlencode($tag),
					3 => RCUtilities::$settings->catalog->defaultSort,
					4 => 1
					
				), TRUE)
				
			);
			
		}
		
	}
	
	private function setDiscounts () {
		
		$now = strtotime('now');
		
		foreach (RCUtilities::$settings->cart->costs as $cost) {
			
			if (
			
				($cost->validity->startEnabled && $now < strtotime($cost->validity->start)) ||
				($cost->validity->endEnabled && $now > strtotime($cost->validity->end)) ||
				(isset($cost->couponRequired) && $cost->couponRequired) ||
				$cost->price > 0
				
			) {
				
				continue;
				
			}
				
			switch ($cost->type) {
				
				case 'product':
				
					foreach ($this->versions as $version) {
			
						if (in_array($version->SKU, $cost->values)) {
						
							$version->discountedPrice = $version->price + (($cost->mode == 'fixed') ? $cost->price : ($version->price * $cost->price / 100));
							
						}
						
					}
				
				break;
				
				case 'tag':
			
					$array = array_intersect($this->tags, $cost->values);
			
					if (!empty($array)) {
				
						foreach ($this->versions as $version) {
							
							$version->discountedPrice = $version->price + (($cost->mode == 'fixed') ? $cost->price : ($version->price * $cost->price / 100));
							
						}
						
					}
					
				break;
				
				case 'category':
			
					$array = array_intersect($this->categories, $cost->values);
					
					if (!empty($array)) {
				
						foreach ($this->versions as $version) {
							
							$version->discountedPrice = $version->price + (($cost->mode == 'fixed') ? $cost->price : ($version->price * $cost->price / 100));
							
						}
						
					}
					
				break;
				
			}
			
		}
		
	}
	
	private function setBasePriceAndTaxesTier () {
		
		$this->basePrice = PHP_INT_MAX;
		
		foreach ($this->versions as $version) {
			
			if (!isset($version->price)) {
				
				$version->price = 0;
				
			}
			
			if (!isset($version->taxesTier)) {
				
				$version->taxesTier = 0;
				
			}
			
			if (isset($version->discountedPrice) && ($this->basePrice > $version->discountedPrice)) {
			
				$this->basePrice = $version->discountedPrice;
				
			}
			
			else if ($this->basePrice > $version->price) {
			
				$this->basePrice = $version->price;
				
			}
			
		}
		
	}
	
	private function addOptions () {
		
		foreach (RCUtilities::$settings->cart->optionSets as $set) {
			
			$match = array_intersect($set->values, (($set->type == 'category') ? $this->categories : (($set->type == 'tag') ? $this->tags : array())));
			
			if (!empty($match)) {
				
				$this->options = array_merge($this->options, $set->options);
				
			}
			
		}
		
	}
	
	public function setVersionAndOptions ($select) {
		
		$result = (object) array(
		
			'name'             => $this->name,
			'productID'        => $this->ID,
			'image'            => $this->images[0]->thumbnailPath,
			'validityID'       => $this->validityID,
			'link'             => $this->link,
			'version'          => '',
			'SKU'              => '',
			'quantity'         => 0,
			'selectedQuantity' => isset($select->quantity[$select->version]) ? $select->quantity[$select->version] : 1,
			'options'          => array(),
			'showOptions'      => FALSE,
			'required'         => array(),
			'price'            => 0,
			'categories'       => array(),
			'tags'             => array(),
			'weight'           => 0,
			'taxesTier'        => 0
		
		);
		
		foreach ($this->versions as $version) {
			
			if ($version->ID == $select->version) {
				
				if ($version->available) {
					
					$result->price = $version->price;
					$result->version = $version->name;
					$result->SKU = $version->SKU;
					$result->quantity = $version->quantity;
					$result->taxable = $version->taxes;
					$result->taxesTier = $version->taxesTier;
					$result->shippable = isset($version->delivery->shipping) && $version->delivery->shipping;
					$result->downloadable = isset($version->delivery->digital) && isset($version->delivery->sources) && isset($version->delivery->sources->AWS) && isset($version->delivery->sources->AWS->buckets) && !empty($version->delivery->sources->AWS->buckets);
					$result->weight = isset($version->weight) ? $version->weight : 0;
					$result->enableStockManagement = isset($version->enableStockManagement) ? $version->enableStockManagement : FALSE;
					$result->stockQuantity = isset($version->enableStockManagement) ? $version->stockQuantity : 0;
					
					if (isset($version->discountedPrice)) {
						
						$result->discountedPrice = $version->discountedPrice;
						
					}
					
					break;
					
				}
				
				return FALSE;
				
			}
			
		}
		
		foreach ($this->options as $option) {
			
			$result->showOptions = TRUE;
		
			$returned_option = (object) array(
						
				'name'   => $option->name,
				'values' => array(),
				
			);
		
			// date, time, textarea, text, number
			if (in_array($option->type, array(
			
				'text',
				'textarea',
				'date',
				'time',
				'number'
			
			))) {
				
				if (!empty($select->options[$option->ID][0])) {
					
					$returned_option->values = $select->options[$option->ID];
					
					if (isset($option->multiplier)) {
						
						$result->price += $select->options[$option->ID][0] * $option->multiplier;
						
					}
					
					
				}
				
				if (
				
					($option->required && empty($select->options[$option->ID][0])) ||
					($option->type == 'number' && ($select->options[$option->ID][0] < $option->min || $select->options[$option->ID][0] > $option->max)) ||
					($option->type == 'date' && ((isset($option->minDateEnabled) && $option->minDateEnabled && RCUtilities::convertLocalizedDate($select->options[$option->ID][0]) < strtotime($option->minDate)) || (isset($option->maxDateEnabled) && $option->maxDateEnabled && RCUtilities::convertLocalizedDate($select->options[$option->ID][0]) > strtotime($option->maxDate))))
					
				) {
					
					$result->required[] = $option->ID;
					
				}
				
			}
			
			// select, checkbox, radio
			else {
				
				$tempRequired = array();
				
				$j = 0;
				
				foreach ($option->values as $i => $value) {
					
					$tempRequired[] = $value->ID;
					
					if (isset($select->options[$option->ID]) && in_array($value->ID, $select->options[$option->ID])) {
					
						$returned_option->values[] = $value->value;
						
						$result->weight += $value->weight;
						$result->price += $value->price;
						
						$j = $i;
						
					}
					
				}
				
				if ($option->required) {
					
					if ($option->type == 'select' && $j == 0) {
						
						$result->required[] = $option->ID;
						
					}
					
					else if (empty($select->options[$option->ID])) {
					
						$result->required = array_merge($result->required, $tempRequired);
						
					}
					
				}
				
			}
			
			if (!empty($returned_option->values)) {
				
				$result->options[] = $returned_option;
				
			}
			
		}
		
		foreach ($this->categories as $category) {
			
			$result->categories[] = $category->name;
			
		}
		
		foreach ($this->tags as $tag) {
			
			$result->tags[] = $tag->name;
			
		}
		
		return $result;
		
	}
	
	public function rel2abs ($relative) {
		
		$absolute = RCUtilities::$settings->general->pageURL;

		if (parse_url($relative, PHP_URL_SCHEME) != '' || empty($absolute)) {
	    	
	    	return $relative;
	    	
	    }
	    
	    if ($relative[0] == '#' || $relative[0] == '?') {
		    
			return $absolute . $relative;

		}
		
		$url = parse_url($absolute);

		$url['path'] = preg_replace('#/[^/]*$#', '', $url['path']);

		if ($relative[0] == '/') {
			
			$url['path'] = '';
			
		}

		$absolute = $url['host'] . ((isset($url['port']) && $url['port'] != 80) ? (':' . $url['port']) : '') . $url['path'] . '/' . $relative;

		$re = array(
		
			'#(/\.?/)#',
			'#/(?!\.\.)[^/]+/\.\./#'
			
		);
		
		for ($i = 1; $i > 0; $absolute = preg_replace($re, '/', $absolute, -1, $i)) {
			
		}
		
		return $url['scheme'] . '://' . $absolute;
		
	}
	
}

?>